local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local PT = AceLibrary("PeriodicTable-2.0")
local L = AceLibrary("AceLocale-2.2"):new("PT2Browser")
local aot
PT2Browser = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0")
PT2Browser.hasIcon = true
PT2Browser:RegisterDB("PT2BrowserDB")
local qualitycolors = {
	[0] = "|cff9d9d9d",
	[1] = "|cffffffff",
	[2] = "|cff1eff00",
	[3] = "|cff0070dd",
	[4] = "|cffa335ee",
	[5] = "|cffff8000" }
function PT2Browser:OnInitialize()
	CreateFrame("GameTooltip","PT2BrowserTooltip",UIParent,"GameTooltipTemplate")
end
function PT2Browser:OnEnable()
	self:RegisterEvent("PT2BRefresh")
end
function PT2Browser:OnTextUpdate()
	if self:IsTextShown() then
		self:SetText(L["PT2"])
	end
	if self:IsIconShown() then
		self:SetIcon("Interface\\Icons\\INV_Holiday_ToW_SpicePotion03")
	else
		self:HideIcon()
	end
end
function PT2Browser:OnTooltipUpdate()
	local uncachedcount = 0
	local cachedcount = 0
	for k,v in pairs(PT.k) do
		for a, b in pairs(v) do
			if type(b) == "string" then
				uncachedcount = uncachedcount+1
			end
		end
	end
	for k,v in pairs(PT.vars.cache) do
		cachedcount = cachedcount+1
	end
	local cat = tablet:AddCategory()
	cat:AddLine('text', string.format(L["PT has %s sets, %s of which are cached."], uncachedcount, cachedcount))
end
function PT2Browser:BuildTable()
	--local timer = GetTime() --!!
	aot = {
		type = 'group',
		args = {
			explodeall = {
				type = 'execute',
				name = L["Explode All (don't do it!)"],
				desc = L["Explode All (don't do it!)"],
				func = function()
					for k,v in pairs(PT.k) do
						for a,b in pairs(PT.k[k]) do
							PT:GetSetTable(a)
						end
					end
				end,
				order = -1,
			}
		}
	}
	for modulename, moduletable in pairs(PT.k) do
		aot.args[modulename] = {
			type = 'group',
			name = modulename,
			desc = modulename,
			args = {}
		}
		for setname, _ in pairs(moduletable) do
			if string.match(setname, " %- ") then
				local prefix, suffix = string.match(setname, "(.+) %- (.+)")
				if not aot.args[modulename].args[prefix] then
					aot.args[modulename].args[prefix] = {
						type = 'group',
						name = prefix,
						desc = prefix,
						args = {}
					}
				end
				aot.args[modulename].args[prefix].args[suffix] = {
					type = 'group',
					name = suffix,
					desc = suffix,
					args = {}
				}
			else
				if not aot.args[modulename].args[setname] then
					aot.args[modulename].args[setname] = {
						type = 'group',
						name = setname,
						desc = setname,
						args = {}
					}
				end
				aot.args[modulename].args[setname].args[setname] = {
					type = 'group',
					name = setname,
					desc = setname,
					args = {}
				}
			end
		end
	end
	--ChatFrame1:AddMessage("time to generate"..GetTime() - timer)
end
function PT2Browser:OnClick(button)
	if button ~= "LeftButton" then return end
	if not aot then PT2Browser:BuildTable() end
	local frame
	if PT2Browser:IsMinimapAttached() then
		frame = FuBarPluginPT2BrowserFrameMinimapButton
	else
		frame = FuBarPluginPT2BrowserFrame
	end
	dewdrop:Open(frame, 'children', function(level, val1, val2, val3)
		dewdrop:FeedAceOptionsTable(aot)
		if level == 4 then
			local setstring
			if val1 == val2 then
				setstring = val1
			else
				setstring = val2.." - "..val1
			end
			if PT.vars.cache[setstring] or type(PT.k[val3][setstring]) == "table" then
				local loaded = true
				if not PT.vars.cache[setstring] then
					for k,v in pairs(PT.k[val3][setstring]) do
						if not PT.vars.cache[v] then
							loaded = false
						end
					end
				end
				if loaded then
					local items
					if type(PT.k[val3][setstring]) == "table" then
						items = PT:GetMultiSetTable(setstring)
					else
						items = PT:GetSetTable(setstring)
					end
					for k,v in pairs(items) do
						local name, link, rarity, _,_,_,_,_,_, texture = GetItemInfo(k)
						local append
						if tonumber(v) then append = "  "..v else append = "" end
						if not link then
							dewdrop:AddLine(
								'text', "|cFFFF2222["..k.."]|r"..append,
								'func', PT2Browser.ItemClick,
								'arg1', self, 'arg2', k,
								'tooltipTitle', "|cFFFF2222"..k,
								'tooltipText', L["Query_Warning"]
							)
						else
							dewdrop:AddLine(
								'text', qualitycolors[rarity].."["..name.."]|r"..append,
								'func', PT2Browser.ItemClick,
								'arg1', self, 'arg2', k,
								'tooltipFunc', PT2Browser.SetTooltip,
								'tooltipArg1', GameTooltip, 'tooltipArg2', k,
								'icon', texture
							)
						end
					end
				else
					dewdrop:AddLine('text', L["Explode"], 'func', function(arg) PT:GetMultiSetTable(arg) end, 'arg1', setstring)
				end
			else
				dewdrop:AddLine('text', L["Explode"], 'func', function(arg) PT:GetSetTable(arg) end, 'arg1', setstring)
			end
		end
	end)
end
function PT2Browser:SetTooltip(arg1)
	GameTooltip_SetDefaultAnchor(GameTooltip, this)
	GameTooltip:SetHyperlink("item:"..arg1..":0:0:0:0:0:0:0")
end
function PT2Browser:ItemClick(id)
	local name, link, rarity, _,_,_,_,_,_, texture = GetItemInfo(id)
	if not name then
		PT2Browser:GetItemData(id)
		PT2Browser:ScheduleEvent("PT2BRefresh", 2)
		return
	end
	if IsControlKeyDown() then
		DressUpItem(id)
	else
		if not ChatFrameEditBox:IsVisible() then
			ChatFrameEditBox:Show()
		end
		ChatFrameEditBox:Insert(link)
	end
end
function PT2Browser:PT2BRefresh()
	dewdrop:Refresh(4)
	dewdrop:Refresh(5)
end
function PT2Browser:GetItemData(id)
	PT2BrowserTooltip:SetOwner(this, "ANCHOR_PRESERVE")
	PT2BrowserTooltip:SetHyperlink("item:" .. id)
	PT2BrowserTooltip:Hide()
end